-- Handles looting specific items. Useful when out farming for specific tradeskill items
-- and you don't want to mess with the loot settings in Lootly/MQ2AutoLoot etc.

local mq = require('mq')
local Write = require('onlyloot/Write')
local lootItems = {}
local isOnlyLootOn, lootlyDisabled, mq2AutoLootDisabled = false, false, false
local running = true

Write.prefix = function() return '\ax[\agOnlyLoot\ax] ' end
Write.loglevel = 'info'
Write.usecolors = false

local function print_usage()
    Write.Info('\agAvailable Commands - ')
    Write.Info('\a-g/onlyloot on\a-t - Enables looting by OnlyLoot, ON by default when the script starts')
    Write.Info('\a-g/onlyloot on\a-t - Disables looting by OnlyLoot')
    Write.Info('\a-g/onlyloot additem <item name>\a-t - Add an item that should be looted.')
    Write.Info('\a-g/onlyloot removeitem <item name>\a-t - Remove an item that should no longer be looted.')
    Write.Info('\a-g/onlyloot log <level>\a-t - Configure log level (info, debug, warn, error, fatal).')
    Write.Info('\a-g/onlyloot stop\a-t - Stops the OnlyLoot script from running')
    Write.Info('\a-g/onlyloot list\a-t - List out all items that will be looted by OnlyLoot')
end

local function handle_loot()
    while mq.TLO.AdvLoot.PCount() > 0 do
        if not mq.TLO.AdvLoot.LootInProgress() then
            local itmName = tostring(mq.TLO.AdvLoot.PList(1).Name())
            if lootItems[itmName] == true then
                Write.Info(string.format('\agLooting %s.', itmName))
                mq.cmd('/advloot personal 1 loot')
            else
                Write.Info(string.format('\agLeaving %s.', itmName))
                mq.cmd('/advloot personal 1 leave')
            end
            mq.delay(500)
        end
    end
end

local function turn_on()
    -- Do nothing if it is already on
    if isOnlyLootOn then return end

    Write.Info('\agEnabling OnlyLoot...')

    -- check for Lootly
    local pids = mq.TLO.Lua.PIDs()
    for pid in string.gmatch(pids, '([^,]+)') do
        local script = mq.TLO.Lua.Script(tonumber(pid))
        if script.Name() == "lootly" then
            Write.Info('\agDisabling Lootly while OnlyLoot is on. We will turn Lootly back on when OnlyLoot ends')
            lootlyDisabled = true
            mq.cmd('/lootly off')
        end
    end
    -- check for MQ2AutoLoot
    if mq.TLO.AutoLoot ~= nil then
        Write.Info('\agDisabling MQ2AutoLoot while OnlyLoot is on. We will turn MQ2AutoLoot back on when OnlyLoot ends')
        mq2AutoLootDisabled = true
        mq.cmd('/AutoLoot turn off')
    end

    isOnlyLootOn = true
end

local function turn_off()
    if not isOnlyLootOn then return end

    Write.Info('\agDisabling OnlyLoot...')
    isOnlyLootOn = false

    -- turn on other loot handlers that were disabled
    if lootlyDisabled then
        Write.Info('\agRe-enabling Lootly now that OnlyLoot is turned off')
        mq.cmd('/lootly on')
    end
    if mq2AutoLootDisabled then
        Write.Info('\agRe-enabling MQ2AutoLoot now that OnlyLoot is turned off')
        mq.cmd('/AutoLoot turn on')
    end
end

local function loot_list()
    Write.Info('The following items will be looted:')
    for k, v in pairs(lootItems) do
        if v then
            Write.Info(string.format('%s', k))
        end
    end
end

local function bind_onlyloot(cmd, val)
    if cmd == 'additem' then
        Write.Info(string.format('\agAdding item to loot list: %s', val))
        lootItems[val] = true
        return
    end
    if cmd == 'removeitem' then
        Write.Info(string.format('\agRemoving item from loot list: %s', val))
        lootItems[val] = false
        return
    end
    if cmd == 'on' then
        turn_on()
        return
    end
    if cmd == 'off' then
        turn_off()
        return
    end
    if cmd == 'stop' then
        running = false
        return
    end
    if cmd == 'list' then
        loot_list()
        return
    end

    -- log levels
    if cmd == 'log' then
        if tValidLogLevels[val] ~= nil then 
            Write.loglevel = val
            Write.Info('\ayLog Level = ' .. Write.loglevel)
        else
            Write.Error(string.format("\arInvalid log level \aw'%s'\ax.", val))
        end
        return
    end

    print_usage()
end

local function setup()
    -- register binds
    mq.bind('/onlyloot', bind_onlyloot)
end

local function in_game() return mq.TLO.MacroQuest.GameState() == 'INGAME' end

local function main()
    turn_on()
    local last_time = os.time()
    while running do
        if in_game() then
            if os.difftime(os.time(), last_time) >= 1 then
                handle_loot()
                last_time = os.time()
            end
            mq.doevents()
        end
        mq.delay(100)
    end
    turn_off()
end

setup()
main()